@@ -58,12 +58,14 @@ typedef struct{
 	const char * header_new_key ;
 	const char * header_new_key_source ;
 	uid_t uid ;
+	int veraCrypt_volume ;
+	int iteration_count ;
 	string_t ( *getKey )( int * ) ;
 }info_t ;
 
 typedef struct{
 	const char * device ;
-	const char * offset ;
+	const char * plain_dm_properties ;
 	const char * mapper_name ;
 	const char * key ;
 	const char * m_point ;
@@ -81,6 +83,8 @@ typedef struct{
 	int tcrypt_system ;
 	const char * const * tcrypt_keyfiles ;
 	unsigned long m_flags ;
+	int use_backup_header ;
+	void * variables ;
 }open_struct_t ;
 
 typedef struct{
@@ -319,6 +323,16 @@ char * zuluCryptLoopDeviceAddress( const char * device ) ;
  */
 char * zuluCryptLoopDeviceAddress_1( const char * device ) ;
 
+/*
+ * this function is defined in ./create_luks.c
+ */
+void zuluCryptDisableMetadataLocking( void ) ;
+
+/*
+ * this function is defined in ./create_luks.c
+ */
+void zuluCryptPrintLogOutPut( void * ) ;
+
 /*
  * this function is defined in ./create_loop_device.c
  */
