@@ -91,7 +91,7 @@ createvolume::createvolume( QWidget * parent ) : QDialog( parent ),m_ui( new Ui:
 	 * for simplicity's sake,lets only show most popular file systems.
 	 */
 
-	m_ui->comboBoxFS->addItems( { "ext4","vfat","ntfs","ext2","ext3","exfat","btrfs" } ) ;
+	m_ui->comboBoxFS->addItems( utility::supportedFileSystems() ) ;
 
 	m_ui->comboBoxVolumeType->clear() ;
 
@@ -227,6 +227,9 @@ void createvolume::volumeType( int s )
 #endif
 		m_ui->comboBoxRNG->setEnabled( true ) ;
 
+		m_ui->label_2->setEnabled( false ) ;
+		m_ui->lineEditPIM->setEnabled( false ) ;
+
 		_disableHidden() ;
 
 		break ;
@@ -235,6 +238,9 @@ void createvolume::volumeType( int s )
 
 		m_ui->comboBoxRNG->setEnabled( false ) ;
 
+		m_ui->label_2->setEnabled( false ) ;
+		m_ui->lineEditPIM->setEnabled( false ) ;
+
 		_disableHidden() ;
 
 		break ;
@@ -243,6 +249,9 @@ void createvolume::volumeType( int s )
 		m_ui->comboBoxRNG->setEnabled( true ) ;
 		m_ui->cbNormalVolume->addItem( tr( "TrueCrypt Keys" ) ) ;
 
+		m_ui->label_2->setEnabled( false ) ;
+		m_ui->lineEditPIM->setEnabled( false ) ;
+
 		_disableHidden() ;
 
 		break ;
@@ -251,6 +260,9 @@ void createvolume::volumeType( int s )
 		m_ui->comboBoxRNG->setEnabled( true ) ;
 		m_ui->cbNormalVolume->addItem( tr( "VeraCrypt Keys" ) ) ;
 
+		m_ui->label_2->setEnabled( true ) ;
+		m_ui->lineEditPIM->setEnabled( true ) ;
+
 		_disableHidden() ;
 
 		break ;
@@ -261,6 +273,9 @@ void createvolume::volumeType( int s )
 		m_ui->cbHiddenVolume->addItem( tr( "TrueCrypt Keys" ) ) ;
 		m_ui->cbNormalVolume->addItem( tr( "TrueCrypt Keys" ) ) ;
 
+		m_ui->label_2->setEnabled( false ) ;
+		m_ui->lineEditPIM->setEnabled( false ) ;
+
 		this->cbHiddenVolume( 0 ) ;
 
 		_enableHidden() ;
@@ -273,6 +288,9 @@ void createvolume::volumeType( int s )
 		m_ui->cbHiddenVolume->addItem( tr( "VeraCrypt Keys" ) ) ;
 		m_ui->cbNormalVolume->addItem( tr( "VeraCrypt Keys" ) ) ;
 
+		m_ui->label_2->setEnabled( true ) ;
+		m_ui->lineEditPIM->setEnabled( true ) ;
+
 		this->cbHiddenVolume( 0 ) ;
 
 		_enableHidden() ;
@@ -383,7 +401,15 @@ void createvolume::setOptions( int e )
 		/*
 		 * crypto options for plain dm-crypt volumes
 		 */
-		options->addItem( "aes.cbc-essiv:256.256.ripemd160" ) ;
+
+		auto s = utility::plainDmCryptOptions() ;
+
+		if( s.isEmpty() ){
+
+			options->addItem( "aes.cbc-essiv:256.256.ripemd160" ) ;
+		}else{
+			options->addItems( s ) ;
+		}
 
 	}else if( _luks() ){
 
@@ -532,41 +558,24 @@ void createvolume::tcryptGui( bool e )
 {
 	m_normalVolume = e ;
 
-	this->disableAll() ;
+	//this->disableAll() ;
 
 	tcrypt::instance( this,true,[ this ]( const QString& key,const QStringList& keyFiles ){
 
-		this->enableAll() ;
+		//this->enableAll() ;
 
 		if( m_normalVolume ){
 
 			m_key = key ;
 			m_keyFiles = keyFiles ;
-			m_ui->lineEditPassphrase1->setEnabled( false ) ;
-			m_ui->pbOpenKeyFile->setEnabled( false ) ;
-
-			auto type = createvolume::createVolumeType( m_ui->comboBoxVolumeType->currentIndex() ) ;
-
-			bool e = type == createvolume::normal_and_hidden_truecrypt || type == createvolume::normal_and_hidden_veracrypt ;
-
-			if( !e ){
-
-				m_ui->lineEditHiddenKey->setEnabled( false ) ;
-				m_ui->lineEditHiddenKey1->setEnabled( false ) ;
-				m_ui->lineEditHiddenSize->setEnabled( false ) ;
-				m_ui->pbHiddenKeyFile->setEnabled( false ) ;
-				m_ui->comboBoxHiddenSize->setEnabled( false ) ;
-				m_ui->cbHiddenVolume->setEnabled( false ) ;
-			}
 		}else{
-			m_ui->lineEditPassphrase1->setEnabled( false ) ;
 			m_hiddenKey = key ;
 			m_hiddenKeyFiles = keyFiles ;
 		}
 
 	},[ this ](){
 
-		this->enableAll() ;
+		//this->enableAll() ;
 
 		bool x = m_ui->cbNormalVolume->currentIndex() == createvolume::normal_truecrypt ;
 		bool y = m_ui->cbNormalVolume->currentIndex() == createvolume::normal_veracrypt ;
@@ -747,6 +756,11 @@ void createvolume::HideUI()
 
 void createvolume::enableAll()
 {
+	auto enable = m_ui->comboBoxVolumeType->currentText().contains( "VeraCrypt" ) ;
+
+	m_ui->label_2->setEnabled( enable ) ;
+	m_ui->lineEditPIM->setEnabled( enable ) ;
+
 	m_ui->labelPassPhrase->setEnabled( true ) ;
 	m_ui->labelVolumePath->setEnabled( true ) ;
 	m_ui->labelRepeatPassPhrase->setEnabled( true ) ;
@@ -807,6 +821,8 @@ void createvolume::enableAll()
 
 void createvolume::disableAll()
 {
+	m_ui->label_2->setEnabled( false ) ;
+	m_ui->lineEditPIM->setEnabled( false ) ;
 	m_ui->labelPassPhrase->setEnabled( false ) ;
 	m_ui->labelVolumePath->setEnabled( false ) ;
 	m_ui->labelRepeatPassPhrase->setEnabled( false ) ;
@@ -960,13 +976,13 @@ void createvolume::pbCreateClicked()
 	case createvolume::normal_truecrypt :
 	case createvolume::normal_and_hidden_truecrypt :
 
-		m_volumeType = "truecrypt" ;
+		m_volumeType = "tcrypt" ;
 
 		break ;
 	case createvolume::normal_veracrypt :
 	case createvolume::normal_and_hidden_veracrypt :
 
-		m_volumeType = "veracrypt" ;
+		m_volumeType = "vcrypt" ;
 
 		break ;
 	default: m_volumeType = "luks" ;
@@ -994,6 +1010,20 @@ void createvolume::pbCreateClicked()
 			case 2 : g += "m" ; break ;
 			case 3 : g += "g" ; break ;
 		}
+
+	}else if( type == createvolume::plain ){
+
+		g += ".0" ;
+
+	}else if( type == createvolume::normal_veracrypt ||
+		  type == createvolume::normal_and_hidden_veracrypt ){
+
+		auto e = m_ui->lineEditPIM->text() ;
+
+		if( !e.isEmpty() ){
+
+			g += "." + e ;
+		}
 	}
 
 	volumePath.replace( "\"","\"\"\"" ) ;
@@ -1104,7 +1134,7 @@ void createvolume::taskFinished_1( int st )
 
 			msg.ShowUIOK( tr( "WARNING!" ),tr( "Volume created successfully but failed to create an external header" ) ) ;
 		}else{
-			bool exit = false ;
+			std::atomic_bool exit( false ) ;
 
 			if( utility::clearVolume( volumePath,&exit,2 * 1024 * 1024,[]( int r ){ Q_UNUSED( r ) ; } ).await() == 0 ){
 
@@ -1127,7 +1157,7 @@ void createvolume::taskFinished( int st )
 
 	auto x = tr( "Volume created successfully." ) ;
 
-	if( m_volumeType.contains( "luks" ) || m_volumeType.contains( "truecrypt" ) || m_volumeType.contains( "veracrypt" ) ){
+	if( utility::containsAtleastOne( m_volumeType,"luks","tcrypt","vcrypt" ) ){
 
 		x += tr( "\nCreating a backup of the \"%1\" volume header is strongly advised.\nPlease read documentation on why this is important." ).arg( m_volumeType ) ;
 	}
