@@ -91,9 +91,16 @@ passwordDialog::passwordDialog( QTableWidget * table,
 	m_ui->pushButtonPlugin->setIcon( QIcon( ":/module.png" ) ) ;
 
 	m_veraCryptVolume = utility::autoSetVolumeAsVeraCrypt( "zuluCrypt-gui" ) ;
-	m_ui->cbVeraCryptVolume->setChecked( m_veraCryptVolume ) ;
 
-	connect( m_ui->cbVeraCryptVolume,SIGNAL( stateChanged( int ) ),this,SLOT( cbVeraCryptVolume( int ) ) ) ;
+	if( m_veraCryptVolume ){
+
+		m_ui->cbVolumeType->setCurrentIndex( 1 ) ;
+		this->cbVolumeType( 1 ) ;
+	}else{
+		m_ui->cbVolumeType->setCurrentIndex( 0 ) ;
+		this->cbVolumeType( 0 ) ;
+	}
+
 	connect( m_ui->PushButtonCancel,SIGNAL( clicked() ),this,SLOT( HideUI() ) ) ;
 	connect( m_ui->PushButtonOpen,SIGNAL( clicked() ),this,SLOT( buttonOpenClicked() ) ) ;
 	connect( m_ui->PushButtonMountPointPath,SIGNAL( clicked() ),this,SLOT( mount_point() ) ) ;
@@ -104,6 +111,8 @@ passwordDialog::passwordDialog( QTableWidget * table,
 	connect( m_ui->pushButtonPlugin,SIGNAL( clicked() ),this,SLOT( pbPlugin() ) ) ;
 	connect( m_ui->pbKeyOption,SIGNAL( clicked() ),this,SLOT( pbKeyOption() ) ) ;
 	connect( m_ui->cbKeyType,SIGNAL( currentIndexChanged( int ) ),this,SLOT( cbActicated( int ) ) ) ;
+	connect( m_ui->cbVolumeType,SIGNAL( currentIndexChanged( int ) ),this,SLOT( cbVolumeType( int ) ) ) ;
+	connect( m_ui->checkBoxVisibleKey,SIGNAL( stateChanged( int ) ),this,SLOT( cbVisibleKeyStateChanged( int ) ) ) ;
 
 	m_ui->PushButtonMountPointPath->setVisible( false ) ;
 	m_ui->pushButtonPassPhraseFromFile->setVisible( false ) ;
@@ -115,16 +124,80 @@ passwordDialog::passwordDialog( QTableWidget * table,
 
 	m_ui->cbKeyType->addItem( tr( "TrueCrypt/VeraCrypt Keys" ) ) ;
 
+	m_ui->pushButtonPlainDmCryptOptions->setMenu( [ this ](){
+
+		auto m = new QMenu( this ) ;
+
+		connect( m,SIGNAL( triggered( QAction * ) ),
+			 this,SLOT( plainDmCryptOption( QAction * ) ) ) ;
+
+		auto s = utility::plainDmCryptOptions() ;
+
+		if( s.isEmpty() ){
+
+			m_plainDmCryptProperty = "aes.cbc-essiv:sha256.256.ripemd160" ;
+			m->addAction( "aes.cbc-essiv:sha256.256.ripemd160" ) ;
+		}else{
+			m_plainDmCryptProperty = s.first() ;
+
+			for( const auto& it : s ){
+
+				m->addAction( it ) ;
+			}
+		}
+
+		return m ;
+	}() ) ;
+
+	m_ui->checkBoxVisibleKey->setToolTip( tr( "Check This Box To Make Password Visible" ) ) ;
+
 	this->setWindowTitle( tr( "Unlock Encrypted Volume" ) ) ;
 
 	this->installEventFilter( this ) ;
 }
 
-void passwordDialog::cbVeraCryptVolume( int state )
+void passwordDialog::plainDmCryptOption( QAction * ac )
 {
-	m_veraCryptVolume = state != Qt::Unchecked ;
+	m_plainDmCryptProperty = ac->text().remove( "&" ) ;
+}
+
+void passwordDialog::cbVolumeType( int e )
+{
+	m_veraCryptVolume = ( e == 1 ) ;
 
 	utility::autoSetVolumeAsVeraCrypt( "zuluCrypt-gui",m_veraCryptVolume ) ;
+
+	m_ui->pushButtonPlainDmCryptOptions->setEnabled( e == 2 ) ;
+
+	m_ui->lineEditVolumeProperty->clear() ;
+
+	m_ui->lineEditVolumeProperty->setToolTip( QString() ) ;
+
+	if( e == 0 ){
+
+		/*
+		 * LUKS,TrueCrypt
+		 */
+		m_ui->labelVolumeProperty->clear() ;
+		m_ui->lineEditVolumeProperty->setEnabled( false ) ;
+
+	}else if( e == 1 ){
+
+		/*
+		 * VeraCrypt volume
+		 */
+		m_ui->labelVolumeProperty->setText( tr( "PIM Value" ) ) ;
+		m_ui->lineEditVolumeProperty->setEnabled( true ) ;
+
+	}else if( e == 2 ){
+
+		/*
+		 * PLAIN dm-crypt with offset
+		 */
+		m_ui->labelVolumeProperty->setText( tr( "Offset" ) ) ;
+		m_ui->lineEditVolumeProperty->setEnabled( true ) ;
+		m_ui->lineEditVolumeProperty->setToolTip( tr( "Offset Will Be In Sectors If The Entry Is Made Up Of Only Digits\nAnd In Bytes If The Entry Ends With \"b\"\nAnd In Kilobytes If The Entry Ends With \"k\"\nAnd In Megabytes If The Entry Ends With \"m\"\nAnd In Terabytes If The Entry Ends With \"t\"" ) ) ;
+	}
 }
 
 bool passwordDialog::eventFilter( QObject * watched,QEvent * event )
@@ -243,6 +316,21 @@ void passwordDialog::ShowUI( QString dev )
 	this->ShowUI( dev,m_point ) ;
 }
 
+void passwordDialog::cbVisibleKeyStateChanged( int s )
+{
+	if( m_ui->cbKeyType->currentIndex() == passwordDialog::key ){
+
+		if( s == Qt::Checked ){
+
+			m_ui->PassPhraseField->setEchoMode( QLineEdit::Normal ) ;
+		}else{
+			m_ui->PassPhraseField->setEchoMode( QLineEdit::Password ) ;
+		}
+
+		m_ui->PassPhraseField->setFocus() ;
+	}
+}
+
 void passwordDialog::ShowUI()
 {
 	this->passphraseOption() ;
@@ -260,6 +348,14 @@ void passwordDialog::mountPointPath( QString path )
 
 void passwordDialog::cbActicated( int e )
 {
+	if( e == passwordDialog::key ){
+
+		m_ui->checkBoxVisibleKey->setEnabled( true ) ;
+	}else{
+		m_ui->checkBoxVisibleKey->setEnabled( false ) ;
+		m_ui->checkBoxVisibleKey->setChecked( false ) ;
+	}
+
 	switch( e ){
 
 		case passwordDialog::key         : return this->passphraseOption() ;
@@ -402,6 +498,7 @@ void passwordDialog::file_path( void )
 	if( !Z.isEmpty() ){
 
 		m_ui->MountPointPath->setText( utility::mountPathPostFix( Z.split( "/" ).last() ) ) ;
+		m_ui->PassPhraseField->setFocus() ;
 	}
 }
 
@@ -485,8 +582,12 @@ void passwordDialog::sendKey( const QString& sockpath )
 
 void passwordDialog::disableAll()
 {
+	m_ui->labelVolumeProperty->setEnabled( false ) ;
+	m_ui->lineEditVolumeProperty->setEnabled( false ) ;
+	m_ui->pushButtonPlainDmCryptOptions->setEnabled( false ) ;
+	m_ui->cbVolumeType->setEnabled( false ) ;
+	m_ui->labelVolumeType->setEnabled( false ) ;
 	m_ui->cbShareMountPoint->setEnabled( false ) ;
-	m_ui->cbVeraCryptVolume->setEnabled( false ) ;
 	m_ui->pushButtonPlugin->setEnabled( false ) ;
 	m_ui->checkBoxReadOnly->setEnabled( false ) ;
 	m_ui->labelMoutPointPath->setEnabled( false ) ;
@@ -502,12 +603,19 @@ void passwordDialog::disableAll()
 	m_ui->PushButtonVolumePath->setEnabled( false ) ;
 	m_ui->pbKeyOption->setEnabled( false ) ;
 	m_ui->cbKeyType->setEnabled( false ) ;
+	m_ui->checkBoxVisibleKey->setEnabled( false ) ;
 }
 
 void passwordDialog::enableAll()
 {
+	auto index = m_ui->cbVolumeType->currentIndex() ;
+
+	m_ui->labelVolumeProperty->setEnabled( index != 0 ) ;
+	m_ui->lineEditVolumeProperty->setEnabled( index != 0 ) ;
+	m_ui->pushButtonPlainDmCryptOptions->setEnabled( index == 2 ) ;
+	m_ui->cbVolumeType->setEnabled( true ) ;
+	m_ui->labelVolumeType->setEnabled( true ) ;
 	m_ui->cbShareMountPoint->setEnabled( true ) ;
-	m_ui->cbVeraCryptVolume->setEnabled( true ) ;
 	m_ui->pushButtonPlugin->setEnabled( true ) ;
 	m_ui->checkBoxReadOnly->setEnabled( true ) ;
 	m_ui->labelMoutPointPath->setEnabled( true ) ;
@@ -522,6 +630,7 @@ void passwordDialog::enableAll()
 	m_ui->pushButtonPassPhraseFromFile->setEnabled( true ) ;
 	m_ui->PushButtonVolumePath->setEnabled( true ) ;
 	m_ui->cbKeyType->setEnabled( true ) ;
+	m_ui->checkBoxVisibleKey->setEnabled( m_ui->cbKeyType->currentIndex() == passwordDialog::key ) ;
 
 	if( m_open_with_path ){
 
@@ -636,6 +745,47 @@ void passwordDialog::openVolume()
 					this->sendKey( keyPath ) ;
 				}
 
+			}else if( r == "network" ){
+
+				auto e = utility::Task::makePath( m_device ) ;
+
+				auto z = QString( "%1 -i -d %2" ).arg( ZULUCRYPTzuluCrypt,e ) ;
+
+				auto& s = utility::Task::run( z ) ;
+
+				auto q = utility::split( s.await().stdOut() ) ;
+
+				if( q.size() < 2 ){
+
+					DialogMsg msg( this ) ;
+
+					msg.ShowUIOK( tr( "ERROR!" ),tr( "Volume is not a LUKS volume" ) ) ;
+
+					m_ui->OpenVolumePath->setFocus() ;
+
+					return this->enableAll() ;
+				}else{
+					this->disableAll() ;
+
+					auto s =  utility::getKeyFromNetwork( q.at( 1 ) ) ;
+
+					if( s.first ){
+
+						m_key = s.second ;
+
+						passtype = "-f" ;
+						keyPath = utility::keyPath() ;
+
+						this->sendKey( keyPath ) ;
+					}else{
+						DialogMsg msg( this ) ;
+
+						msg.ShowUIOK( tr( "ERROR!" ),tr( "Failed to get a key from the network" ) ) ;
+
+						m_ui->OpenVolumePath->setFocus() ;
+						return this->enableAll() ;
+					}
+				}
 			}else{
 				passtype = "-G" ;
 				keyPath  = r ;
@@ -648,7 +798,7 @@ void passwordDialog::openVolume()
 
 		this->sendKey( keyPath ) ;
 	}else{
-		qDebug() << "Error: uncaught condition" ;
+		utility::debug() << "Error: uncaught condition" ;
 	}
 
 	QString a = ZULUCRYPTzuluCrypt ;
@@ -675,7 +825,26 @@ void passwordDialog::openVolume()
 
 	if( m_veraCryptVolume ){
 
-		exe += " -t vcrypt" ;
+		auto e = m_ui->lineEditVolumeProperty->text() ;
+
+		if( e.isEmpty() ){
+
+			exe += " -t vcrypt" ;
+		}else {
+			exe += " -t vcrypt." + e ;
+		}
+	}else{
+		if( m_ui->cbVolumeType->currentIndex() == 2 ){
+
+			auto e = m_ui->lineEditVolumeProperty->text() ;
+
+			if( e.isEmpty() ){
+
+				exe += " -t " + m_plainDmCryptProperty + ".0" ;
+			}else {
+				exe += " -t " + m_plainDmCryptProperty + "." + e ;
+			}
+		}
 	}
 
 	if( m_ui->cbShareMountPoint->isChecked() ){
