@@ -1,4 +1,4 @@
-﻿/*
+/*
  *
  *  Copyright ( c ) 2011-2015
  *  name : Francis Banyikwa
@@ -17,7 +17,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "utility.h"
 #include <iostream>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -29,6 +28,7 @@
 #include <QDir>
 #include <pwd.h>
 #include <grp.h>
+#include <termios.h>
 
 #include <memory>
 
@@ -47,10 +47,11 @@
 #include <QProcessEnvironment>
 #include <unistd.h>
 #include <pwd.h>
-
+#include <QApplication>
 #include <QEvent>
 #include <QKeyEvent>
-
+#include <QMetaObject>
+#include <QtNetwork/QLocalSocket>
 #include "../zuluCrypt-cli/constants.h"
 #include "../zuluCrypt-cli/bin/bash_special_chars.h"
 #include "version.h"
@@ -62,6 +63,10 @@
 #include "../plugins/plugins.h"
 #include "plugin.h"
 #include "install_prefix.h"
+#include "utility.h"
+#include "executablesearchpaths.h"
+#include "networkAccessManager.hpp"
+#include "zuluPolkit.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -81,7 +86,234 @@
 #include "reuse_mount_point.h"
 #include "share_mount_prefix_path.h"
 
+#include "zuluPolkit.h"
+
+struct jsonResult
+{
+	bool finished ;
+	int exitCode ;
+	int exitStatus ;
+	QByteArray stdError ;
+	QByteArray stdOut ;
+};
+
+#if POLKIT_SUPPORT
+
+#include "json.h"
+
+static QByteArray _json_command( const QByteArray& cookie,
+				 const QByteArray& password,
+				 const QString& exe )
+{
+	nlohmann::json json ;
+
+	json[ "cookie" ]   = cookie.constData() ;
+	json[ "password" ] = password.constData() ;
+	json[ "command" ]  = exe.toLatin1().constData() ;
+
+	return json.dump().c_str() ;
+}
+
+static jsonResult _json_result( const QByteArray& e )
+{
+	try{
+		auto json = nlohmann::json::parse( e.constData() ) ;
+
+		return { json[ "finished" ].get< bool >(),
+			 json[ "exitCode" ].get< int >(),
+			 json[ "exitStatus" ].get< int >(),
+			 json[ "stdError" ].get< std::string >().c_str(),
+			 json[ "stdOut" ].get< std::string >().c_str() } ;
+	}catch( ... ){
+		return { false,255,255,"","" } ;
+	}
+}
+
+#else
+
+static QByteArray _json_command( const QByteArray& cookie,
+				 const QByteArray& password,
+				 const QString& exe )
+{
+	Q_UNUSED( cookie ) ;
+	Q_UNUSED( password ) ;
+	Q_UNUSED( exe ) ;
+
+	return QByteArray() ;
+}
+
+static jsonResult _json_result( const QByteArray& e )
+{
+	Q_UNUSED( e ) ;
+	return { false,255,255,"","" } ;
+}
+
+#endif
+
 static int staticGlobalUserId = -1 ;
+static QByteArray _cookie ;
+
+::Task::future< utility::Task >& utility::Task::run( const QString& exe,bool e )
+{
+	return utility::Task::run( exe,-1,e ) ;
+}
+
+::Task::future< utility::Task >& utility::Task::run( const QString& exe,int s,bool e )
+{
+	return ::Task::run< utility::Task >( [ = ](){
+
+		auto env = QProcessEnvironment::systemEnvironment() ;
+
+		return utility::Task( exe,s,env,_cookie,[](){ umask( 0 ) ; },e ) ;
+	} ) ;
+}
+
+void utility::Task::execute( const QString& exe,int waitTime,
+			     const QProcessEnvironment& env,
+			     const QByteArray& password,
+			     const std::function< void() >& f,
+			     bool polkit )
+{
+	class Process : public QProcess{
+	public:
+		Process( const std::function< void() >& f ) : m_function( f )
+		{
+		}
+	protected:
+		void setupChildProcess()
+		{
+			m_function() ;
+		}
+	private:
+		std::function< void() > m_function ;
+	} p( f ) ;
+
+	p.setProcessEnvironment( env ) ;
+
+	if( polkit && utility::useZuluPolkit() ){
+
+		QLocalSocket s ;
+
+		s.connectToServer( utility::helperSocketPath() ) ;
+
+		for( int i = 0 ; ; i++ ){
+
+			if( s.waitForConnected() ){
+
+				break ;
+
+			}else if( i == 10 ){
+
+				utility::debug() << "ERROR: Failed To Start Helper Application" ;
+				return ;
+			}else{
+				utility::debug() << s.errorString() ;
+
+				utility::Task::suspendForOneSecond() ;
+			}
+		}
+
+		s.write( _json_command( _cookie,password,exe ) ) ;
+
+		s.waitForBytesWritten() ;
+
+		s.waitForReadyRead() ;
+
+		auto e = _json_result( s.readAll() ) ;
+
+		m_finished   = e.finished ;
+		m_exitCode   = e.exitCode ;
+		m_exitStatus = e.exitStatus ;
+		m_stdError   = e.stdError ;
+		m_stdOut     = e.stdOut ;
+	}else{
+		p.start( exe ) ;
+
+		if( !password.isEmpty() ){
+
+			p.waitForStarted() ;
+
+			p.write( password + '\n' ) ;
+
+			p.closeWriteChannel() ;
+		}
+
+		m_finished   = p.waitForFinished( waitTime ) ;
+		m_exitCode   = p.exitCode() ;
+		m_exitStatus = p.exitStatus() ;
+		m_stdOut     = p.readAllStandardOutput() ;
+		m_stdError   = p.readAllStandardError() ;
+	}
+}
+
+void utility::startHelperExecutable( QObject * obj,const QString& arg,const char * slot,const char * slot1 )
+{
+	if( !utility::useZuluPolkit() ){
+
+		QMetaObject::invokeMethod( obj,slot,Q_ARG( bool,true ),Q_ARG( QString,QString() ) ) ;
+
+		return ;
+	}
+
+	QFile f( "/dev/urandom" ) ;
+
+	f.open( QIODevice::ReadOnly ) ;
+
+	_cookie = f.read( 16 ).toHex() ;
+
+	auto exe = utility::executableFullPath( "pkexec" ) ;
+
+	if( !exe.isEmpty() ){
+
+		exe = QString( "%1 %2 %3 fork" ).arg( exe,zuluPolkitPath,utility::helperSocketPath() ) ;
+
+		::Task::exec( [ = ](){
+
+			auto e = utility::Task::run( exe,false ).get() ;
+
+			QMetaObject::invokeMethod( obj,
+						   slot,
+						   Q_ARG( bool,e.success() ),
+						   Q_ARG( QString,arg ) ) ;
+		} ) ;
+	}else{
+		DialogMsg().ShowUIOK( QObject::tr( "ERROR" ),QObject::tr( "Failed to locate pkexec executable" ) ) ;
+		QMetaObject::invokeMethod( obj,slot1 ) ;
+	}
+}
+
+QString utility::helperSocketPath()
+{
+	auto z = utility::homePath() + "/.zuluCrypt-socket/" ;
+
+	z += QCoreApplication::applicationName() + ".polkit.socket" ;
+
+	return z ;
+}
+
+bool utility::useZuluPolkit()
+{
+	return POLKIT_SUPPORT ;
+}
+
+void utility::quitHelper()
+{
+	auto e = utility::helperSocketPath() ;
+
+	if( utility::pathExists( e ) ){
+
+		QLocalSocket s ;
+
+		s.connectToServer( e ) ;
+
+		if( s.waitForConnected() ){
+
+			s.write( _json_command( _cookie,"","exit" ) ) ;
+
+			s.waitForBytesWritten() ;
+		}
+	}
+}
 
 bool utility::reUseMountPointPath()
 {
@@ -95,7 +327,7 @@ bool utility::reUseMountPoint()
 
 void utility::setUID( int uid )
 {
-	if( utility::userIsRoot() ){
+	if( utility::userIsRoot() || utility::useZuluPolkit() ){
 
 		staticGlobalUserId = uid ;
 	}
@@ -116,6 +348,18 @@ int utility::getUserID()
 	}
 }
 
+int utility::getGUID( int uid )
+{
+	auto e = getpwuid( uid ) ;
+
+	if( e ){
+
+		return e->pw_gid ;
+	}else{
+		return uid ;
+	}
+}
+
 QString utility::getStringUserID()
 {
 	return QString::number( utility::getUserID() ) ;
@@ -184,6 +428,11 @@ int utility::startApplication( const char * appName,std::function<int()> start )
 	}
 }
 
+void utility::startApplication( QObject * s,const char * e )
+{
+	QMetaObject::invokeMethod( s,e,Qt::QueuedConnection ) ;
+}
+
 void utility::keySend( const QString& path,const QByteArray& key )
 {
 	::Task::exec( [ path,key ](){
@@ -222,16 +471,13 @@ void utility::createPlugInMenu( QMenu * menu,const QString& a,const QString& b,c
 
 	l.append( a ) ;
 
-	if( utility::notRunningInMixedMode() ){
+	if( LXQt::Wallet::backEndIsSupported( LXQt::Wallet::BackEnd::libsecret ) ){
 
-		if( LXQt::Wallet::backEndIsSupported( LXQt::Wallet::BackEnd::libsecret ) ){
+		l.append( b ) ;
+	}
+	if( LXQt::Wallet::backEndIsSupported( LXQt::Wallet::BackEnd::kwallet ) ){
 
-			l.append( b ) ;
-		}
-		if( LXQt::Wallet::backEndIsSupported( LXQt::Wallet::BackEnd::kwallet ) ){
-
-			l.append( c ) ;
-		}
+		l.append( c ) ;
 	}
 
 	if( addPlugIns ){
@@ -248,6 +494,8 @@ void utility::createPlugInMenu( QMenu * menu,const QString& a,const QString& b,c
 			e.removeOne( "keyring" ) ;
 			e.removeOne( "kwallet" ) ;
 		}
+
+		e.append( "network" ) ;
 	}
 
 	menu->clear() ;
@@ -279,7 +527,7 @@ void utility::createPlugInMenu( QMenu * menu,const QString& a,const QString& b,c
 
 		if( r.success() ){
 
-			const auto& s = r.output() ;
+			const auto& s = r.stdOut() ;
 
 			int i = 0 ;
 
@@ -310,7 +558,7 @@ void utility::createPlugInMenu( QMenu * menu,const QString& a,const QString& b,c
 
 		if( r.success() ){
 
-			QString uuid = r.output() ;
+			QString uuid = r.stdOut() ;
 			uuid.remove( "\n" ) ;
 
 			if( uuid == "UUID=\"\"" ){
@@ -325,69 +573,45 @@ void utility::createPlugInMenu( QMenu * menu,const QString& a,const QString& b,c
 	} ) ;
 }
 
-static bool _execute_process( const QString& m,const QString& exe,const QString& env,int uid )
+void utility::dropPrivileges( int uid )
 {
-	Q_UNUSED( env ) ;
+	if( uid == -1 ){
 
-	if( exe.startsWith( "/" ) && utility::pathExists( exe ) ){
+		uid = utility::getUserID() ;
+	}
 
-		return utility::Task( exe + " " + utility::Task::makePath( m ),[ & ](){
+	if( uid != -1 ){
 
-			return QProcessEnvironment() ;
+		auto id = getpwuid( uid ) ;
 
-		}(),[ uid ](){
+		if( id ){
 
-			if( uid != -1 ){
+			setenv( "LOGNAME",id->pw_name,1 ) ;
+			setenv( "HOME",id->pw_dir,1 ) ;
+			setenv( "USER",id->pw_name,1 ) ;
+		}
 
-				auto id = getpwuid( uid ) ;
-
-				if( id ){
-
-					setenv( "LOGNAME",id->pw_name,1 ) ;
-					setenv( "HOME",id->pw_dir,1 ) ;
-					setenv( "USER",id->pw_name,1 ) ;
-				}
-
-				Q_UNUSED( setgid( uid ) ) ;
-				Q_UNUSED( setgroups( 1,reinterpret_cast< const gid_t * >( &uid ) ) ) ;
-				Q_UNUSED( setegid( uid ) ) ;
-				Q_UNUSED( setuid( uid ) ) ;
-
-			}
-
-			auto path = [](){
-
-				auto e = getenv( "PATH" ) ;
-
-				QString s = "/usr/local/bin:/usr/bin:/bin:" ;
-
-				if( e ){
-					return e + QString( ":" ) + s ;
-				}else{
-					return s ;
-				}
-			}() ;
-
-			setenv( "PATH",path.toLatin1().constData(),1 ) ;
-
-		} ).success() ;
-	}else{
-		return false ;
+		Q_UNUSED( setgid( uid ) ) ;
+		Q_UNUSED( setgroups( 1,reinterpret_cast< const gid_t * >( &uid ) ) ) ;
+		Q_UNUSED( setegid( uid ) ) ;
+		Q_UNUSED( setuid( uid ) ) ;
 	}
 }
 
-::Task::future<bool>& utility::openPath( const QString& path,const QString& opener,const QString& env )
+::Task::future<bool>& utility::openPath( const QString& path,const QString& opener )
 {
-	return ::Task::run<bool>( [ env,path,opener ](){
+	return ::Task::run<bool>( [ = ](){
 
-		return _execute_process( path,opener,env,utility::getUID() ) == false ;
+		auto e = opener + " " + utility::Task::makePath( path ) ;
+
+		return utility::Task::run( e,false ).get().failed() ;
 	} ) ;
 }
 
 void utility::openPath( const QString& path,const QString& opener,
-			const QString& env,QWidget * obj,const QString& title,const QString& msg )
+			QWidget * obj,const QString& title,const QString& msg )
 {
-	openPath( path,opener,env ).then( [ title,msg,obj ]( bool failed ){
+	openPath( path,opener ).then( [ title,msg,obj ]( bool failed ){
 
 		if( failed && obj ){
 
@@ -521,7 +745,7 @@ static bool _writeToVolume( int fd,const char * buffer,unsigned int bufferSize )
 	return write( fd,buffer,bufferSize ) != -1 ;
 }
 
-::Task::future< int >& utility::clearVolume( const QString& volume,bool * exit,size_t volumeSize,std::function< void( int ) > function )
+::Task::future< int >& utility::clearVolume( const QString& volume,std::atomic_bool * exit,size_t volumeSize,std::function< void( int ) > function )
 {
 	return ::Task::run<int>( [ volume,exit,volumeSize,function ](){
 
@@ -719,24 +943,42 @@ QString utility::applicationName()
 	return "zuluCrypt" ;
 }
 
-bool utility::pathIsReadable( const QString& path )
+bool utility::pathIsReadable( const QString& path,bool isFolder )
 {
-	utility::fileHandle h ;
+	QFileInfo s( path ) ;
 
-	return h.open( path ) ;
+	if( isFolder ){
+
+		return s.isReadable() && s.isDir() ;
+	}else{
+		return s.isReadable() && s.isFile() ;
+	}
 }
 
-bool utility::pathIsWritable( const QString& path )
+bool utility::pathIsWritable( const QString& path,bool isFolder )
 {
-	utility::fileHandle h ;
+	QFileInfo s( path ) ;
 
-	if( h.open( path,false ) ){
+	if( isFolder ){
 
-		h.unlink() ;
-
-		return true ;
+		return s.isWritable() && s.isDir() ;
 	}else{
+		return s.isWritable() && s.isFile() ;
+	}
+}
+
+bool utility::configDirectoriesAreNotWritable( QWidget * w )
+{
+	auto a = utility::homePath() + "/.zuluCrypt" ;
+	auto b = utility::homePath() + "/.zuluCrypt-socket" ;
+
+	if( utility::pathIsWritable( a ) && utility::pathIsWritable( b ) ){
+
 		return false ;
+	}else{
+		auto e = QObject::tr( "\"%1\" Folder Must Be Writable.\n\"%2\" Folder Must Also Be Writable." ).arg( a,b ) ;
+		DialogMsg( w ).ShowUIOK( QObject::tr( "ERROR" ),e ) ;
+		return true ;
 	}
 }
 
@@ -921,7 +1163,16 @@ bool utility::pathExists( const QString& path )
 
 bool utility::canCreateFile( const QString& path )
 {
-	return utility::pathIsWritable( path ) ;
+	auto s = path ;
+
+	auto e = s.lastIndexOf( '/' ) ;
+
+	if( e != -1 ){
+
+		s.truncate( e ) ;
+	}
+
+	return utility::pathIsWritable( s ) ;
 }
 
 QString utility::resolvePath( const QString& path )
@@ -963,6 +1214,21 @@ QString utility::executableFullPath( const QString& f )
 {
 	QString e = f ;
 
+	if( e.startsWith( "/" ) ){
+
+		auto s =  QDir( f ).canonicalPath() ;
+
+		for( const auto& it : utility::executableSearchPaths() ){
+
+			if( s.startsWith( it ) ){
+
+				return s ;
+			}
+		}
+
+		return QString() ;
+	}
+
 	if( e == "ecryptfs" ){
 
 		e = "ecryptfs-simple" ;
@@ -970,14 +1236,7 @@ QString utility::executableFullPath( const QString& f )
 
 	QString exe ;
 
-	auto q = { "/usr/local/bin/",
-		   "/usr/local/sbin/",
-		   "/usr/bin/",
-		   "/usr/sbin/",
-		   "/bin/",
-		   "/sbin/" } ;
-
-	for( const auto& it : q ){
+	for( const auto& it : utility::executableSearchPaths() ){
 
 		exe = it + e ;
 
@@ -1030,12 +1289,7 @@ QString utility::cmdArgumentValue( const QStringList& l,const QString& arg,const
 		}
 	}
 
-	if( defaulT == "xdg-open" ){
-
-		return _absolute_exe_path( defaulT ) ;
-	}else{
-		return defaulT ;
-	}
+	return defaulT ;
 }
 
 static QString _device_id_to_partition_id( const QString& id )
@@ -1208,9 +1462,14 @@ void utility::readFavorites( QMenu * m,bool truncate,bool showFolders )
 
 			m->addAction( _add_action( it ) ) ;
 		}else{
-			if( !utility::pathPointsToAFolder( utility::split( it,'\t' ).first() ) ){
+			auto e = utility::split( it,'\t' ).first() ;
 
-				m->addAction( _add_action( it ) ) ;
+			if( utility::pathExists( e ) ){
+
+				if( !utility::pathPointsToAFolder( e ) ){
+
+					m->addAction( _add_action( it ) ) ;
+				}
 			}
 		}
 	}
@@ -1241,6 +1500,12 @@ bool utility::userHasGoodVersionOfWhirlpool()
 
 void utility::licenseInfo( QWidget * parent )
 {
+	QString s ;
+#if POLKIT_SUPPORT
+	s = "\nPolkit Support: YES" ;
+#else
+	s = "\nPolkit Support: NO" ;
+#endif
 	QString license = QString( "%1\n\n\
 This program is free software: you can redistribute it and/or modify \
 it under the terms of the GNU General Public License as published by \
@@ -1250,10 +1515,8 @@ the Free Software Foundation, either version 2 of the License, or \
 This program is distributed in the hope that it will be useful,\
 but WITHOUT ANY WARRANTY; without even the implied warranty of \
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License\n\
-along with this program.  If not, see <http://www.gnu.org/licenses/>." ).arg( VERSION_STRING ) ;
+GNU General Public License for more details." ).arg( VERSION_STRING + s ) ;
+
 
 	DialogMsg m( parent ) ;
 	m.ShowUIInfo( QObject::tr( "about zuluCrypt" ),false,license ) ;
@@ -1457,7 +1720,7 @@ void utility::saveFont( const QFont& Font )
 
 bool utility::runningInMixedMode()
 {
-	return utility::userIsRoot() && utility::getUID() != -1 ;
+	return utility::useZuluPolkit() ;
 }
 
 bool utility::notRunningInMixedMode()
@@ -1935,3 +2198,268 @@ void utility::createFolderPath( const QString& e )
 	utility::changePathOwner( e ) ;
 	utility::changePathPermissions( e,0777 ) ;
 }
+
+QStringList utility::plainDmCryptOptions()
+{
+	const QByteArray _options = R"(aes.cbc-essiv:sha256.256.ripemd160
+aes.cbc-essiv:sha256.256.sha256
+aes.cbc-essiv:sha256.256.sha512
+aes.cbc-essiv:sha256.256.sha1
+aes.cbc-essiv:sha256.512.ripemd160
+aes.cbc-essiv:sha256.512.sha256
+aes.cbc-essiv:sha256.512.sha512
+aes.cbc-essiv:sha256.512.sha1
+aes.xts-plain64.256.ripemd160
+aes.xts-plain64.256.sha256
+aes.xts-plain64.256.sha512
+aes.xts-plain64.256.sha1
+aes.xts-plain64.512.ripemd160
+aes.xts-plain64.512.sha256
+aes.xts-plain64.512.sha512
+aes.xts-plain64.512.sha1)" ;
+
+	QFile f( utility::homePath() + "/.zuluCrypt/plainDmCryptOptions" ) ;
+
+	if( !f.exists() ){
+
+		if( f.open( QIODevice::WriteOnly ) ){
+
+			f.write( _options ) ;
+
+			utility::changePathOwner( f ) ;
+			utility::changePathPermissions( f,0666 ) ;
+
+			f.close() ;
+		}
+	}
+
+	if( f.open( QIODevice::ReadOnly ) ){
+
+		return utility::split( f.readAll() ) ;
+	}
+
+	return QStringList() ;
+}
+
+QStringList utility::supportedFileSystems()
+{
+	QFile f( utility::homePath() + "/.zuluCrypt/supportedFileSystems" ) ;
+
+	if( !f.exists() ){
+
+		if( f.open( QIODevice::WriteOnly ) ){
+
+			f.write( "ext4\nvfat\nntfs\next2\next3\nexfat\nbtrfs" ) ;
+
+			utility::changePathOwner( f ) ;
+			utility::changePathPermissions( f,0666 ) ;
+
+			f.close() ;
+		}
+	}
+
+	if( f.open( QIODevice::ReadOnly ) ){
+
+		return utility::split( f.readAll() ) ;
+	}
+
+	return { "ext4","vfat","ntfs","ext2","ext3","exfat","btrfs" } ;
+}
+
+std::pair< bool,QByteArray > utility::getKeyFromNetwork( const QString& e )
+{
+	QFile f( utility::homePath() + "/.zuluCrypt/network" ) ;
+
+	if( !f.open( QIODevice::ReadOnly ) ){
+
+		return { false,"" } ;
+	}
+
+	QUrl url ;
+
+	QByteArray data = "device=" + utility::split( e,' ' ).last().toLatin1() ;
+
+	QByteArray host ;
+
+	for( const auto& it : utility::split( f.readAll() ) ){
+
+		if( it.startsWith( "url=" ) ){
+
+			url.setUrl( it.toLatin1().constData() + 4 ) ;
+
+		}else if( it.startsWith( "host=" ) ){
+
+			host = it.toLatin1().constData() + 5 ;
+		}else{
+			data += "&" + it ;
+		}
+	}
+
+	QNetworkRequest n( url ) ;
+
+	n.setRawHeader( "Host",host ) ;
+	n.setRawHeader( "Content-Type","application/x-www-form-urlencoded" ) ;
+
+	NetworkAccessManager m ;
+
+	auto s = m.post( n,data ) ;
+
+	if( s->error() == QNetworkReply::NoError ){
+
+		return { true,s->readAll() } ;
+	}else{
+		return { false,"" } ;
+	}
+}
+
+void utility::setHDPI( const QString& e )
+{
+	Q_UNUSED( e ) ;
+
+#if QT_VERSION >= 0x050600
+
+	QApplication::setAttribute( Qt::AA_EnableHighDpiScaling ) ;
+
+	QFile f( utility::homePath() + "/.zuluCrypt/" + e + ".scaleFactor" ) ;
+
+	if( !f.exists() ){
+
+		f.open( QIODevice::WriteOnly ) ;
+		f.write( "1" ) ;
+		f.close() ;
+	}
+
+	if( f.open( QIODevice::ReadOnly ) ){
+
+		qputenv( "QT_SCALE_FACTOR",f.readAll().replace( "\n","" ) ) ;
+	}
+#endif
+}
+
+bool utility::platformIsLinux()
+{
+	return true ;
+}
+
+bool utility::platformIsOSX()
+{
+	return false ;
+}
+
+QStringList utility::executableSearchPaths()
+{
+	return ::executableSearchPaths::values() ;
+}
+
+QString utility::executableSearchPaths( const QString& e )
+{
+	if( e.isEmpty() ){
+
+		return utility::executableSearchPaths().join( ":" ) ;
+	}else{
+		return e + ":" + utility::executableSearchPaths().join( ":" ) ;
+	}
+}
+
+static inline bool _terminalEchoOff( struct termios * old,struct termios * current )
+{
+	if( tcgetattr( 1,old ) != 0 ){
+
+		return false ;
+	}
+
+	*current = *old;
+	current->c_lflag &= ~ECHO;
+
+	if( tcsetattr( 1,TCSAFLUSH,current ) != 0 ){
+
+		return false ;
+	}else{
+		return true ;
+	}
+}
+
+QString utility::readPassword( bool addNewLine )
+{
+	std::cout << "Password: " << std::flush ;
+
+	struct termios old ;
+	struct termios current ;
+
+	_terminalEchoOff( &old,&current ) ;
+
+	QString s ;
+	int e ;
+
+	int m = 1024 ;
+
+	for( int i = 0 ; i < m ; i++ ){
+
+		e = std::getchar() ;
+
+		if( e == '\n' || e == -1 ){
+
+			break ;
+		}else{
+			s += static_cast< char >( e ) ;
+		}
+	}
+
+	tcsetattr( 1,TCSAFLUSH,&old ) ;
+
+	if( addNewLine ){
+
+		std::cout << std::endl ;
+	}
+
+	return s ;
+}
+
+QString utility::fileManager()
+{
+	QFile f( utility::homePath() + "/.zuluCrypt/FileManager" ) ;
+
+	if( !f.exists() ){
+
+		f.open( QIODevice::WriteOnly | QIODevice::Truncate ) ;
+
+		f.write( "xdg-open" ) ;
+
+		f.close() ;
+	}
+
+	if( !f.open( QIODevice::ReadOnly ) ){
+
+		return "xdg-open" ;
+	}else{
+		return utility::split( f.readAll() ).first() ;
+	}
+}
+
+void utility::setFileManager( const QString& e )
+{
+	QFile f( utility::homePath() + "/.zuluCrypt/FileManager" ) ;
+
+	f.open( QIODevice::WriteOnly | QIODevice::Truncate ) ;
+
+	if( e.isEmpty() ){
+
+		f.write( "xdg-open" ) ;
+	}else{
+		f.write( e.toLatin1() ) ;
+	}
+}
+
+QProcessEnvironment utility::systemEnvironment()
+{
+	auto e = QProcessEnvironment::systemEnvironment() ;
+
+	e.insert( "CRYFS_NO_UPDATE_CHECK","TRUE" ) ;
+	e.insert( "CRYFS_FRONTEND","noninteractive" ) ;
+
+	e.insert( "LANG","C" ) ;
+
+	e.insert( "PATH",utility::executableSearchPaths( e.value( "PATH" ) ) ) ;
+
+	return e ;
+}
