@@ -43,11 +43,12 @@
 #include <memory>
 #include <array>
 #include <utility>
-
+#include <QtNetwork/QLocalSocket>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <pwd.h>
 
 #include <blkid/blkid.h>
 
@@ -61,13 +62,14 @@
 #include <fcntl.h>
 
 #include <iostream>
+#include <atomic>
 
 namespace utility
 {
 	class debug
 	{
 	public:
-		debug( bool stdout = true ) : m_stdout( stdout )
+		debug( bool s = true ) : m_stdout( s )
 		{
 		}
 
@@ -107,6 +109,24 @@ namespace utility
 
 			return utility::debug( m_stdout ) ;
 		}
+
+		utility::debug operator<<( const QStringList& e )
+		{
+			if( m_stdout ){
+
+				for( const auto& it : e ){
+
+					std::cout << it.toLatin1().constData() << std::endl ;
+				}
+			}else{
+				for( const auto& it : e ){
+
+					std::cerr << it.toLatin1().constData() << std::endl ;
+				}
+			}
+
+			return utility::debug( m_stdout ) ;
+		}
 	private:
 		bool m_stdout ;
 	};
@@ -163,6 +183,7 @@ namespace utility
 
 	int getUID() ;
 	int getUserID() ;
+	int getGUID( int uid ) ;
 
 	QString getStringUserID() ;
 	QString appendUserUID( const QString& ) ;
@@ -172,21 +193,24 @@ namespace utility
 	void changePathOwner( const T& f )
 	{
 		int uid = utility::getUID() ;
+		int gid = utility::getGUID( uid ) ;
+
 		int fd = f.handle() ;
 
 		if( uid != -1 && fd != -1 ){
 
-			if( fchown( fd,uid,uid ) ){;}
+			if( fchown( fd,uid,gid ) ){;}
 		}
 	}
 
 	static inline void changePathOwner( const char * path )
 	{
 		int uid = utility::getUID() ;
+		int gid = utility::getGUID( uid ) ;
 
 		if( uid != -1 ){
 
-			if( chown( path,uid,uid ) ){;}
+			if( chown( path,uid,gid ) ){;}
 		}
 	}
 
@@ -217,6 +241,7 @@ namespace utility
 	};
 
 	int startApplication( const char * appName,std::function<int()> ) ;
+	void startApplication( QObject *,const char * ) ;
 
 	wallet getKey( LXQt::Wallet::Wallet&,const QString& keyID,
 		       const QString& app = QString() ) ;
@@ -235,6 +260,8 @@ namespace utility
 	void autoOpenFolderOnMount( const QString&,bool ) ;
 	bool autoOpenFolderOnMount( const QString& ) ;
 
+	QProcessEnvironment systemEnvironment() ;
+
 	bool userBelongsToGroup( const char * groupname ) ;
 	bool runningInMixedMode( void ) ;
 	bool notRunningInMixedMode( void ) ;
@@ -242,8 +269,14 @@ namespace utility
 	QStringList readFavorites( void ) ;
 	void readFavorites( QMenu *,bool = false,bool = true ) ;
 	void removeFavoriteEntry( const QString& ) ;
+	void setHDPI( const QString& ) ;
 	bool pathExists( const QString& ) ;
 	bool canCreateFile( const QString& ) ;
+	bool useZuluPolkit( void ) ;
+	void startHelperExecutable( QObject *,const QString&,const char *,const char * ) ;
+	void dropPrivileges( int = -1 ) ;
+	QString fileManager( void ) ;
+	void setFileManager( const QString& ) ;
 	QString prettyfySpaceUsage( quint64 ) ;
 	QString resolvePath( const QString& ) ;
 	QString hashPath( const QByteArray& ) ;
@@ -261,13 +294,15 @@ namespace utility
 	QString sharedMountPointPath( const QString& ) ;
 	bool pathPointsToAFile( const QString& ) ;
 	bool pathPointsToAFolder( const QString& ) ;
+	bool configDirectoriesAreNotWritable( QWidget * ) ;
 	QString localizationLanguage( const QString& ) ;
 	QString localizationLanguagePath( const QString& ) ;
 	void setLocalizationLanguage( const QString&,const QString& ) ;
 	QString walletName( void ) ;
 	QString applicationName( void ) ;
-	bool pathIsReadable( const QString& ) ;
-	bool pathIsWritable( const QString& ) ;
+	QString readPassword( bool = true ) ;
+	bool pathIsReadable( const QString&,bool isFolder = true ) ;
+	bool pathIsWritable( const QString&,bool isFolder = true ) ;
 	bool setOpenVolumeReadOnly( QWidget * parent,bool check,const QString& app ) ;
 	bool getOpenVolumeReadOnlyOption( const QString& app ) ;
 	QString keyPath( void ) ;
@@ -277,12 +312,21 @@ namespace utility
 	QStringList split( const QString&,char token = '\n' ) ;
 	QStringList split( const QByteArray&,char token = '\n' ) ;
 	QStringList directoryList( const QString& ) ;
+	QStringList plainDmCryptOptions( void ) ;
+	QStringList supportedFileSystems( void ) ;
 	bool userHasGoodVersionOfWhirlpool( void ) ;
 	void licenseInfo( QWidget * ) ;
 	void showTrayIcon( QAction *,QSystemTrayIcon *,bool = true ) ;
 	void trayProperty( QSystemTrayIcon *,bool = true ) ;
 	void createHomeFolder( void ) ;
 	void createFolderPath( const QString& ) ;
+	bool platformIsLinux( void ) ;
+	bool platformIsOSX( void ) ;
+	QStringList executableSearchPaths( void ) ;
+	QString executableSearchPaths( const QString& ) ;
+	QString helperSocketPath( void ) ;
+	void quitHelper( void ) ;
+	std::pair< bool,QByteArray > getKeyFromNetwork( const QString& ) ;
 
 	QString powerOffCommand( void ) ;
 
@@ -308,13 +352,13 @@ namespace utility
 	QFont getFont( QWidget * ) ;
 	void saveFont( const QFont& ) ;
 
-	::Task::future< int >& clearVolume( const QString& volumePath,bool * exit,size_t size,std::function< void( int ) > ) ;
+	::Task::future< int >& clearVolume( const QString& volumePath,std::atomic_bool * exit,size_t size,std::function< void( int ) > ) ;
 	::Task::future< int >& exec( const QString& ) ;
 	::Task::future< QStringList >& luksEmptySlots( const QString& volumePath ) ;
 	::Task::future< QString >& getUUIDFromPath( const QString& ) ;
-	::Task::future< bool >& openPath( const QString& path,const QString& opener,const QString& env = QString() ) ;
+	::Task::future< bool >& openPath( const QString& path,const QString& opener ) ;
 
-	void openPath( const QString& path,const QString& opener,const QString& env,QWidget *,const QString&,const QString& ) ;
+	void openPath( const QString& path,const QString& opener,QWidget *,const QString&,const QString& ) ;
 }
 
 namespace utility
@@ -535,6 +579,10 @@ namespace utility
 	class Task
 	{
 	public :
+		static ::Task::future< utility::Task >& run( const QString& exe,bool e ) ;
+
+		static ::Task::future< utility::Task >& run( const QString& exe,int,bool e ) ;
+
 		static ::Task::future< utility::Task >& run( const QString& exe )
 		{
 			return ::Task::run< utility::Task >( [ exe ](){ return utility::Task( exe ) ; } ) ;
@@ -577,25 +625,25 @@ namespace utility
 		{
 		}
 		Task( const QString& exe,int waitTime = -1,const QProcessEnvironment& env = QProcessEnvironment(),
-		      const QByteArray& password = QByteArray(),const std::function< void() >& f = [](){} )
+		      const QByteArray& password = QByteArray(),const std::function< void() >& f = [](){},bool e = true )
 		{
-			this->execute( exe,waitTime,env,password,f ) ;
+			this->execute( exe,waitTime,env,password,f,e ) ;
 		}
-		Task( const QString& exe,const QProcessEnvironment& env,const std::function< void() >& f )
+		Task( const QString& exe,const QProcessEnvironment& env,const std::function< void() >& f,bool e = true )
 		{
-			this->execute( exe,-1,env,QByteArray(),f ) ;
+			this->execute( exe,-1,env,QByteArray(),f,e ) ;
 		}
 		QStringList splitOutput( char token ) const
 		{
-			return utility::split( m_data,token ) ;
+			return utility::split( m_stdOut,token ) ;
 		}
-		void output( const QByteArray& r )
+		void stdOut( const QByteArray& r )
 		{
-			m_data = r ;
+			m_stdOut = r ;
 		}
-		const QByteArray& output() const
+		const QByteArray& stdOut() const
 		{
-			return m_data ;
+			return m_stdOut ;
 		}
 		const QByteArray& stdError() const
 		{
@@ -624,46 +672,11 @@ namespace utility
 		bool ok() const
 		{
 			return this->splitOutput( '\n' ).size() > 12 ;
-		}
+		}	
 	private:
 		void execute( const QString& exe,int waitTime,const QProcessEnvironment& env,
-			      const QByteArray& password,const std::function< void() >& f )
-		{
-			class Process : public QProcess{
-			public:
-				Process( const std::function< void() >& f ) : m_function( f )
-				{
-				}
-			protected:
-				void setupChildProcess()
-				{
-					m_function() ;
-				}
-			private:
-				std::function< void() > m_function ;
-			} p( f ) ;
-
-			p.setProcessEnvironment( env ) ;
-
-			p.start( exe ) ;
-
-			if( !password.isEmpty() ){
-
-				p.waitForStarted() ;
-
-				p.write( password + '\n' ) ;
-
-				p.closeWriteChannel() ;
-			}
-
-			m_finished   = p.waitForFinished( waitTime ) ;
-			m_exitCode   = p.exitCode() ;
-			m_exitStatus = p.exitStatus() ;
-			m_data       = p.readAll() ;
-			m_stdError   = p.readAllStandardError() ;
-		}
-
-		QByteArray m_data ;
+			      const QByteArray& password,const std::function< void() >& f,bool ) ;
+		QByteArray m_stdOut ;
 		QByteArray m_stdError ;
 
 		int m_exitCode ;
@@ -733,4 +746,36 @@ namespace utility
 		QString m_warning = tr( "Please be patient as unlocking a VeraCrypt volume may take a very long time.\n\n" ) ;
 	};
 }
+
+namespace utility
+{
+	static inline ::Task::future< void > * startTask( std::function< void() > task,
+							  std::function< void() > continuation )
+	{
+		auto& e = ::Task::run( std::move( task ) ) ;
+
+		e.then( std::move( continuation ) ) ;
+
+		return std::addressof( e ) ;
+	}
+
+	static inline void stopTask( ::Task::future< void > * task,
+				      std::function< void() >& function )
+	{
+		if( task ){
+
+			auto e = task->first_thread() ;
+
+			if( e->isRunning() ){
+
+				e->terminate() ;
+			}else{
+				function() ;
+			}
+		}else{
+			function() ;
+		}
+	}
+}
+
 #endif // MISCFUNCTIONS_H
