@@ -1,4 +1,4 @@
-/*
+﻿/*
  *
  *  Copyright ( c ) 2011-2015
  *  name : Francis Banyikwa
@@ -53,6 +53,7 @@
 #include "luksdeletekey.h"
 #include "createvolume.h"
 #include "createfile.h"
+#include "filemanager.h"
 #include "createkeyfile.h"
 #include "favorites.h"
 #include "cryptoinfo.h"
@@ -97,23 +98,40 @@ void zuluCrypt::languageMenu( QAction * ac )
 	m_ui->retranslateUi( this ) ;
 }
 
-void zuluCrypt::setUpApp( const QString& volume )
+void zuluCrypt::helperStarted( bool e,const QString& volume )
 {
-	this->setLocalizationLanguage( true ) ;
+	if( e ){
 
-	m_ui = new Ui::zuluCrypt ;
+		this->setLocalizationLanguage( true ) ;
 
-	this->setupUIElements() ;
-	this->setupConnections() ;
-	this->initFont() ;
-	this->initKeyCombo() ;
-	this->initTray() ;
-	this->info() ;
-	this->autoUpdateCheck() ;
-	this->setLocalizationLanguage( false ) ;
-	this->updateVolumeList( volume ) ;
+		m_ui = new Ui::zuluCrypt ;
 
-	m_mountInfo.start() ;
+		this->setupUIElements() ;
+		this->setupConnections() ;
+		this->initFont() ;
+		this->initKeyCombo() ;
+		this->initTray() ;
+		this->info() ;
+		this->autoUpdateCheck() ;
+		this->setLocalizationLanguage( false ) ;
+		this->updateVolumeList( volume ) ;
+
+		m_mountInfo->start() ;
+
+		if( m_startHidden ){
+
+			m_trayIcon.show() ;
+		}else{
+			this->setVisible( true ) ;
+			this->show() ;
+			this->raise() ;
+			this->setWindowState( Qt::WindowActive ) ;
+		}
+	}else{
+		DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "Failed To Start Helper Application.\n\"org.zulucrypt.zulupolkit.policy\" polkit file is either misconfigured or zuluPolkit executable could not be found." ) ) ;
+
+		this->closeApplication() ;
+	}
 }
 
 void zuluCrypt::updateVolumeList( const QString& volume )
@@ -128,7 +146,7 @@ void zuluCrypt::updateVolumeList( const QString& volume )
 
 		if( r.success() ){
 
-			return r.output() ;
+			return r.stdOut() ;
 		}else{
 			return QByteArray() ;
 		}
@@ -151,7 +169,9 @@ void zuluCrypt::updateVolumeList( QString volume,QString r )
 
 				if( q.startsWith( "crypto_LUKS" ) ){
 
-					z.replace( 2,"luks" ) ;
+					auto s = q ;
+
+					z.replace( 2,s.replace( "crypto_","" ).toLower()  ) ;
 				}else{
 					auto e = q ;
 
@@ -193,26 +213,17 @@ void zuluCrypt::initFont()
 	this->setUserFont( utility::getFont( this ) ) ;
 }
 
-void zuluCrypt::raiseWindow()
-{
-	if( m_startHidden ){
-
-		m_trayIcon.setVisible( true ) ;
-	}else{
-		this->setVisible( true ) ;
-		this->show() ;
-		this->raise() ;
-		this->setWindowState( Qt::WindowActive ) ;
-	}
-}
-
-void zuluCrypt::raiseWindow( QString device )
+void zuluCrypt::raiseWindow( const QString& device )
 {
 	this->setVisible( true ) ;
 	this->show() ;
 	this->raise() ;
 	this->setWindowState( Qt::WindowActive ) ;
-	this->ShowPasswordDialog( device,device.split( "/" ).last() ) ;
+
+	if( !device.isEmpty() ){
+
+		this->ShowPasswordDialog( device,device.split( "/" ).last() ) ;
+	}
 }
 
 void zuluCrypt::start()
@@ -225,26 +236,37 @@ void zuluCrypt::start()
 	 * runs.
 	 */
 
-	auto l         = QCoreApplication::arguments() ;
+	const auto l   = QCoreApplication::arguments() ;
 
-	auto e         = utility::cmdArgumentValue( l,"-d" ) ;
-
-	m_openPath     = utility::cmdArgumentValue( l,"-m","xdg-open" ) ;
+	m_openPath     = utility::cmdArgumentValue( l,"-m",utility::fileManager() ) ;
 	m_startHidden  = l.contains( "-e" ) ;
 
-	m_env          = utility::cmdArgumentValue( l,"-z","" ) ;
+	if( utility::useZuluPolkit() ){
 
-	utility::setUID( utility::cmdArgumentValue( l,"-K","-1" ).toInt() ) ;
+		utility::setUID( getuid() ) ;
+	}else{
+		utility::setUID( utility::cmdArgumentValue( l,"-K","-1" ).toInt() ) ;
+	}
+
+	auto s = utility::homePath() + "/.zuluCrypt-socket" ;
 
 	utility::createHomeFolder() ;
+	utility::createFolderPath( s ) ;
 
-	oneinstance::instance( this,"zuluCrypt-gui.socket","raiseWindow",e,[ this,e ]( QObject * instance ){
+	if( utility::configDirectoriesAreNotWritable( this ) ){
 
-		this->setUpApp( e ) ;
+		return this->closeApplication() ;
+	}
 
-		connect( instance,SIGNAL( raise() ),this,SLOT( raiseWindow() ) ) ;
-		connect( instance,SIGNAL( raiseWithDevice( QString ) ),this,SLOT( raiseWindow( QString ) ) ) ;
-	} ) ;
+	oneinstance::instance( this,
+			       s + "/zuluCrypt-gui.socket",
+			       utility::cmdArgumentValue( l,"-d" ),
+			       [ this ]( const QString& e ){ utility::startHelperExecutable( this,
+											     e,
+											     "helperStarted",
+											     "closeApplication" ) ; },
+			       [ this ]( int s ){ this->closeApplication( s ) ;	},
+			       [ this ]( const QString& e ){ this->raiseWindow( e ) ; } ) ;
 }
 
 void zuluCrypt::initTray()
@@ -361,6 +383,7 @@ void zuluCrypt::setupConnections()
 	connect( m_ui->actionOpen_zuluCrypt_pdf,SIGNAL( triggered() ),this,SLOT( openpdf() ) ) ;
 	connect( m_ui->actionCheck_For_Update,SIGNAL( triggered() ),this,SLOT( updateCheck() ) ) ;
 	connect( m_ui->actionContact_Info,SIGNAL( triggered() ),this,SLOT( cinfo() ) ) ;
+	connect( m_ui->actionSet_File_Manager,SIGNAL( triggered() ),this,SLOT( setFileManager() ) ) ;
 
 	connect( this,SIGNAL( closeVolume( QTableWidgetItem *,int ) ),this,SLOT( closeAll( QTableWidgetItem *,int ) ) ) ;
 
@@ -377,23 +400,17 @@ void zuluCrypt::setupConnections()
 	connect( m_ui->actionBackup_header,SIGNAL( triggered() ),this,SLOT( volumeHeaderBackUp() ) ) ;
 	connect( m_ui->actionRestore_header,SIGNAL( triggered() ),this,SLOT( volumeRestoreHeader() ) ) ;
 
-	if( utility::runningInMixedMode() ){
+	using wbe = LXQt::Wallet::BackEnd ;
 
-		m_ui->actionManage_volumes_in_gnome_wallet->setEnabled( false ) ;
-		m_ui->actionManage_volumes_in_kde_wallet->setEnabled( false ) ;
-	}else{
-		using wbe = LXQt::Wallet::BackEnd ;
+	auto a = LXQt::Wallet::backEndIsSupported( wbe::libsecret ) ;
+	auto b = LXQt::Wallet::backEndIsSupported( wbe::kwallet ) ;
 
-		auto a = LXQt::Wallet::backEndIsSupported( wbe::libsecret ) ;
-		auto b = LXQt::Wallet::backEndIsSupported( wbe::kwallet ) ;
-
-		m_ui->actionManage_volumes_in_gnome_wallet->setEnabled( a ) ;
-		m_ui->actionManage_volumes_in_kde_wallet->setEnabled( b ) ;
-	}
+	m_ui->actionManage_volumes_in_gnome_wallet->setEnabled( a ) ;
+	m_ui->actionManage_volumes_in_kde_wallet->setEnabled( b ) ;
 
 	connect( m_ui->menuOptions,SIGNAL( aboutToShow() ),this,SLOT( optionMenuAboutToShow() ) ) ;
 
-	connect( &m_mountInfo,SIGNAL( gotEvent() ),this,SLOT( updateVolumeList() ) ) ;
+	connect( m_mountInfo.get(),SIGNAL( gotEvent() ),this,SLOT( updateVolumeList() ) ) ;
 
 	m_language_menu = [ this ](){
 
@@ -546,7 +563,7 @@ void zuluCrypt::closeAllVolumes()
 {
 	m_ui->tableWidget->setEnabled( false ) ;
 
-	m_mountInfo.announceEvents( false ) ;
+	m_mountInfo->announceEvents( false ) ;
 
 	Task::await( [ this ](){
 
@@ -585,7 +602,7 @@ void zuluCrypt::closeAllVolumes()
 
 	} ) ;
 
-	m_mountInfo.announceEvents( true ) ;
+	m_mountInfo->announceEvents( true ) ;
 
 	m_ui->tableWidget->setEnabled( true ) ;
 }
@@ -658,7 +675,14 @@ void zuluCrypt::quitApplication()
 
 void zuluCrypt::closeApplication()
 {
-	m_mountInfo.stop()() ;
+	utility::quitHelper() ;
+	m_mountInfo->stop()() ;
+}
+
+void zuluCrypt::closeApplication( int s )
+{
+	Q_UNUSED( s ) ;
+	m_mountInfo->stop()() ;
 }
 
 void zuluCrypt::trayClicked( QSystemTrayIcon::ActivationReason e )
@@ -787,7 +811,7 @@ void zuluCrypt::volume_property()
 
 		if( r.success() ){
 
-			auto data = r.output() ;
+			auto data = r.stdOut() ;
 
 			return QString( " %1" ).arg( QString( data.mid( data.indexOf( '\n' ) + 2 ) ) ) ;
 		}else{
@@ -893,7 +917,7 @@ void zuluCrypt::openFolder( const QString& path )
 	auto x = tr( "WARNING!" ) ;
 	auto y = tr( "Could not open mount point because \"%1\" tool does not appear to be working correctly").arg( m_openPath ) ;
 
-	utility::openPath( path,m_openPath,m_env,this,x,y ) ;
+	utility::openPath( path,m_openPath,this,x,y ) ;
 }
 
 void zuluCrypt::openpdf()
@@ -905,15 +929,15 @@ void zuluCrypt::openpdf()
 
 	if( utility::pathExists( e ) ){
 
-		utility::openPath( e,m_openPath,m_env,this,x,y ) ;
+		utility::openPath( e,m_openPath,this,x,y ) ;
 	}else{
 		e += ".gz" ;
 
 		if( utility::pathExists( e ) ){
 
-			utility::openPath( e,m_openPath,m_env,this,x,y ) ;
+			utility::openPath( e,m_openPath,this,x,y ) ;
 		}else{
-			utility::openPath( PDF_PATH,m_openPath,m_env,this,x,y ) ;
+			utility::openPath( PDF_PATH,m_openPath,this,x,y ) ;
 		}
 	}
 }
@@ -952,7 +976,7 @@ void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
 
 	m.addSeparator() ;
 
-	if( m_ui->tableWidget->item( item->row(),2 )->text() == "luks" ){
+	if( m_ui->tableWidget->item( item->row(),2 )->text().startsWith( "luks" ) ){
 
 		m.addSeparator() ;
 
@@ -989,7 +1013,7 @@ void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
 
 	m.addSeparator() ;
 
-	connect( m.addAction( tr( "Close" ) ),SIGNAL( triggered() ),this,SLOT( close() ) ) ;
+	connect( m.addAction( tr( "Unmount" ) ),SIGNAL( triggered() ),this,SLOT( close() ) ) ;
 
 	if( clicked ){
 
@@ -1073,6 +1097,11 @@ void zuluCrypt::close()
 	this->closeStatusErrorMessage( r ) ;
 }
 
+void zuluCrypt::setFileManager()
+{
+	fileManager::instance( this,[ this ]( const QString& e ){ m_openPath = e ; } ) ;
+}
+
 void zuluCrypt::volumeRestoreHeader()
 {
 	managevolumeheader::instance( this ).restoreHeader() ;
@@ -1197,6 +1226,11 @@ void zuluCrypt::decryptFile( const QString& e )
 
 zuluCrypt::~zuluCrypt()
 {
+	if( !m_ui ){
+
+		return ;
+	}
+
 	auto q = m_ui->tableWidget ;
 
 	const auto& r = this->window()->geometry() ;
