@@ -31,7 +31,7 @@
 
 #include "bin_path.h"
 #include "../zuluCrypt-gui/dialogmsg.h"
-#include "../zuluCrypt-gui/task.h"
+#include "task.h"
 #include "../zuluCrypt-cli/constants.h"
 #include "plugin_path.h"
 #include "../zuluCrypt-gui/utility.h"
@@ -39,7 +39,6 @@
 #include "mountoptions.h"
 #include "../zuluCrypt-gui/tcrypt.h"
 #include "zulumounttask.h"
-#include "../zuluCrypt-gui/task.h"
 #include "zulumounttask.h"
 #include "siritask.h"
 #include "veracrypt_support.h"
@@ -78,6 +77,8 @@ keyDialog::keyDialog( QWidget * parent,
 		msg = tr( "Mount An Encrypted Volume In \"%1\"").arg( m_path ) ;
 	}
 
+	m_ui->checkBoxVisibleKey->setToolTip( tr( "Check This Box To Make Password Visible" ) ) ;
+
 	this->setWindowTitle( msg ) ;
 
 	m_ui->lineEditMountPoint->setText( m_path ) ;
@@ -113,6 +114,7 @@ keyDialog::keyDialog( QWidget * parent,
 	connect( m_ui->pbOpenMountPoint,SIGNAL( clicked() ),this,SLOT( pbMountPointPath() ) ) ;
 	connect( m_ui->checkBoxOpenReadOnly,SIGNAL( stateChanged( int ) ),this,SLOT( cbMountReadOnlyStateChanged( int ) ) ) ;
 	connect( m_ui->cbKeyType,SIGNAL( currentIndexChanged( int ) ),this,SLOT( cbActicated( int ) ) ) ;
+	connect( m_ui->checkBoxVisibleKey,SIGNAL( stateChanged( int ) ),this,SLOT( cbVisibleKeyStateChanged( int ) ) ) ;
 
 	m_ui->pbOpenMountPoint->setVisible( false ) ;
 
@@ -170,6 +172,8 @@ keyDialog::keyDialog( QWidget * parent,
 
 	m_veraCryptWarning.setWarningLabel( m_ui->veraCryptWarning ) ;
 
+	m_ui->pbkeyOption->setVisible( false ) ;
+	m_ui->checkBoxVisibleKey->setVisible( true ) ;
 	this->installEventFilter( this ) ;
 }
 
@@ -274,8 +278,24 @@ void keyDialog::pbMountPointPath()
 	}
 }
 
+void keyDialog::cbVisibleKeyStateChanged( int s )
+{
+	if( m_ui->cbKeyType->currentIndex() == keyDialog::Key ){
+
+		if( s == Qt::Checked ){
+
+			m_ui->lineEditKey->setEchoMode( QLineEdit::Normal ) ;
+		}else{
+			m_ui->lineEditKey->setEchoMode( QLineEdit::Password ) ;
+		}
+
+		m_ui->lineEditKey->setFocus() ;
+	}
+}
+
 void keyDialog::enableAll()
 {
+	m_ui->checkBoxVisibleKey->setEnabled( m_ui->cbKeyType->currentIndex() == keyDialog::Key ) ;
 	m_ui->checkBoxVeraCryptVolume->setEnabled( true ) ;
 	m_ui->pbOptions->setEnabled( !m_encryptedFolder ) ;
 	m_ui->label_2->setEnabled( true ) ;
@@ -298,6 +318,7 @@ void keyDialog::enableAll()
 
 void keyDialog::disableAll()
 {
+	m_ui->checkBoxVisibleKey->setEnabled( false ) ;
 	m_ui->checkBoxVeraCryptVolume->setEnabled( false ) ;
 	m_ui->cbKeyType->setEnabled( false ) ;
 	m_ui->pbOptions->setEnabled( false ) ;
@@ -452,10 +473,10 @@ void keyDialog::encryptedFolderMount()
 
 	auto ro = m_ui->checkBoxOpenReadOnly->isChecked() ;
 
-	auto& e = siritask::encryptedFolderMount( { m_path,m,m_key,QString(),QString(),
-						    QString(),ro,m_success } ) ;
+	auto e = siritask::encryptedFolderMount( { m_path,m,m_key,QString(),QString(),
+						   QString(),ro,m_success } ).await() ;
 
-	switch( e.await() ){
+	switch( e.status() ){
 
 	case siritask::status::success :
 
@@ -481,6 +502,11 @@ void keyDialog::encryptedFolderMount()
 		msg.ShowUIOK( tr( "ERROR" ),tr( "Failed to unlock an ecryptfs volume.\nWrong password entered" ) ) ;
 		break;
 
+	case siritask::status::ecryptfsIllegalPath :
+
+		msg.ShowUIOK( tr( "ERROR" ),tr( "A Space Character Is Not Allowed In Paths When Using Ecryptfs Backend And Polkit" ) ) ;
+		break;
+
 	case siritask::status::securefs :
 
 		msg.ShowUIOK( tr( "ERROR" ),tr( "Failed to unlock a securefs volume.\nWrong password entered" ) ) ;
@@ -522,11 +548,9 @@ void keyDialog::encryptedFolderMount()
 		break;
 
 	case siritask::status::backendFail :
-
-		msg.ShowUIOK( tr( "ERROR" ),tr( "Failed to unlock the volume.\nBackend not responding" ) ) ;
-		break;
 	default:
-		msg.ShowUIOK( tr( "ERROR" ),tr( "Failed to unlock the volume.\nNot supported volume encountered" ) ) ;
+
+		msg.ShowUIOK( tr( "ERROR" ),e.msg() ) ;
 		break;
 	}
 
@@ -779,9 +803,9 @@ void keyDialog::openVolume()
 
 		if( m_veraCryptPIMValue > 0 ){
 
-			exe += " -t veracrypt." + QString::number( m_veraCryptPIMValue ) + " " + m ;
+			exe += " -t vcrypt." + QString::number( m_veraCryptPIMValue ) + " " + m ;
 		}else{
-			exe += " -t veracrypt " + m ;
+			exe += " -t vcrypt " + m ;
 		}
 	}else{
 		exe += " " + m ;
@@ -819,7 +843,7 @@ void keyDialog::openVolume()
 		}else{
 			if( this->errorNotFound( r ) ){
 
-				QString z = s.output() ;
+				QString z = s.stdOut() ;
 
 				z.replace( tr( "ERROR: " ),"" ) ;
 
@@ -849,6 +873,9 @@ void keyDialog::openVolume()
 
 void keyDialog::cbActicated( int e )
 {
+	m_ui->pbkeyOption->setVisible( e != keyDialog::Key ) ;
+	m_ui->checkBoxVisibleKey->setVisible( e == keyDialog::Key ) ;
+
 	switch( e ){
 
 		case keyDialog::Key        : return this->key() ;
