@@ -23,12 +23,15 @@
 
 int main( int argc,char * argv[] )
 {
+	utility::setHDPI( "zuluMount" ) ;
+
 	QApplication a( argc,argv ) ;
 
 	return utility::startApplication( "zuluMount",[ & ](){
 
 		zuluMount e ;
-		e.Show() ;
+
+		utility::startApplication( &e,"Show" ) ;
 
 		return a.exec() ;
 	} ) ;
