@@ -56,7 +56,7 @@
 #include "../zuluCrypt-gui/utility.h"
 #include "siritask.h"
 #include "zulumounttask.h"
-#include "../zuluCrypt-gui/task.h"
+#include "task.h"
 #include "../zuluCrypt-gui/checkforupdates.h"
 #include "../zuluCrypt-gui/favorites.h"
 
@@ -69,13 +69,20 @@ static bool _encrypted_folder( const QString& e,bool f = true )
 
 zuluMount::zuluMount( QWidget * parent ) :
 	QWidget( parent ),
-	m_mountInfo( monitor_mountinfo::instance( this,true,[ this ](){ this->quitApplication() ; } ) ),
-	m_events( events::instance( this,m_mountInfo.stop() ) )
+	m_mountInfo( monitor_mountinfo::instance( this,true,[ this ](){ QCoreApplication::quit() ; } ) ),
+	m_events( events::instance( this,m_mountInfo->stop() ) )
 {
 }
 
-void zuluMount::setUpApp( const QString& volume )
+void zuluMount::helperStarted( bool start,const QString& volume )
 {
+	if( !start ){
+
+		DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "Failed To Start Helper Application.\n\"org.zulucrypt.zulupolkit.policy\" polkit file is either misconfigured or zuluPolkit executable could not be found." ) ) ;
+
+		return this->closeApplication() ;
+	}
+
 	this->setLocalizationLanguage( true ) ;
 
 	m_ui = new Ui::zuluMount ;
@@ -277,7 +284,7 @@ void zuluMount::setUpApp( const QString& volume )
 		auto ac = new QAction( tr( "Check For Update" ),this ) ;
 
 		m_actionPair.append( { ac,"Check For Update" } ) ;
-		
+
 		connect( ac,SIGNAL( triggered() ),this,SLOT( updateCheck() ) ) ;
 
 		return ac ;
@@ -354,6 +361,11 @@ void zuluMount::setUpApp( const QString& volume )
 	}
 
 	this->autoUpdateCheck() ;
+
+	if( !m_startHidden ){
+
+		this->raiseWindow() ;
+	}
 }
 
 void zuluMount::showTrayGUI()
@@ -539,21 +551,23 @@ bool zuluMount::autoOpenFolderOnMount( void )
 
 void zuluMount::startAutoMonitor()
 {
-	m_mountInfo.start() ;
-	m_events.start() ;
+	m_mountInfo->start() ;
+	m_events->start() ;
 }
 
 /*
- * This should be the only function that closes the application
+ * Below two functions should be the only functions that closes the application
  */
 void zuluMount::closeApplication()
 {
-	m_events.stop() ;
+	utility::quitHelper() ;
+	m_events->stop() ;
 }
 
-void zuluMount::quitApplication()
+void zuluMount::closeApplication( int s )
 {
-	QCoreApplication::quit() ;
+	Q_UNUSED( s ) ;
+	m_events->stop() ;
 }
 
 void zuluMount::autoMountVolume( volumeProperty * q )
@@ -667,51 +681,52 @@ void zuluMount::itemEntered( QTableWidgetItem * item )
 	item->setToolTip( z ) ;
 }
 
-void zuluMount::startGUI()
+void zuluMount::raiseWindow( const QString& volume )
 {
-	if( !m_startHidden ){
-		this->raiseWindow() ;
+	this->setVisible( true ) ;
+	this->raise() ;
+	this->show() ;
+	this->setWindowState( Qt::WindowActive ) ;
+
+	if( !volume.isEmpty() ){
+
+		this->showMoungDialog( volume ) ;
 	}
 }
 
-void zuluMount::raiseWindow()
-{
-	this->setVisible( true ) ;
-	this->raise() ;
-	this->show() ;
-	this->setWindowState( Qt::WindowActive ) ;
-}
-
-void zuluMount::raiseWindow( QString volume )
-{
-	this->setVisible( true ) ;
-	this->raise() ;
-	this->show() ;
-	this->setWindowState( Qt::WindowActive ) ;
-	this->showMoungDialog( volume ) ;
-}
-
 void zuluMount::Show()
 {
 	auto l = QCoreApplication::arguments() ;
 
 	m_startHidden  = l.contains( "-e" ) ;
-	m_folderOpener = utility::cmdArgumentValue( l,"-m","xdg-open" ) ;
-	m_env          = utility::cmdArgumentValue( l,"-z","" ) ;
+	m_folderOpener = utility::cmdArgumentValue( l,"-m",utility::fileManager() ) ;
 
-	utility::setUID( utility::cmdArgumentValue( l,"-K","-1" ).toInt() ) ;
+	if( utility::useZuluPolkit() ){
+
+		utility::setUID( getuid() ) ;
+	}else{
+		utility::setUID( utility::cmdArgumentValue( l,"-K","-1" ).toInt() ) ;
+	}
+
+	auto s = utility::homePath() + "/.zuluCrypt-socket" ;
 
 	utility::createHomeFolder() ;
+	utility::createFolderPath( s ) ;
 
-	auto volume = utility::cmdArgumentValue( l,"-d" ) ;
+	if( utility::configDirectoriesAreNotWritable( this ) ){
 
-	oneinstance::instance( this,"zuluMount-gui.socket","startGUI",volume,[ this,volume ]( QObject * instance ){
+		return this->closeApplication() ;
+	}
 
-		connect( instance,SIGNAL( raise() ),this,SLOT( raiseWindow() ) ) ;
-		connect( instance,SIGNAL( raiseWithDevice( QString ) ),this,SLOT( raiseWindow( QString ) ) ) ;
-
-		this->setUpApp( volume ) ;
-	} ) ;
+	oneinstance::instance( this,
+			       s + "/zuluMount-gui.socket",
+			       utility::cmdArgumentValue( l,"-d" ),
+			       [ this ]( const QString& e ){ utility::startHelperExecutable( this,
+											     e,
+											     "helperStarted",
+											     "closeApplication" ) ; },
+			       [ this ]( int s ){ this->closeApplication( s ) ;	},
+			       [ this ]( const QString& e ){ this->raiseWindow( e ) ; } ) ;
 }
 
 void zuluMount::cryfsVolumeProperties()
@@ -876,7 +891,7 @@ void zuluMount::showContextMenu( QTableWidgetItem * item,bool itemClicked )
 					 this,SLOT( slotOpenFolder() ) ) ;
 				connect( m.addAction( tr( "Open Shared Folder" ) ),SIGNAL( triggered() ),
 					 this,SLOT( slotOpenSharedFolder() ) ) ;
-			}			
+			}
 
 			m.addSeparator() ;
 
@@ -971,7 +986,7 @@ void zuluMount::openMountPoint( const QString& m_point )
 	auto x = tr( "Warning" ) ;
 	auto y = tr( "Could not open mount point because \"%1\" tool does not appear to be working correctly").arg( m_folderOpener ) ;
 
-	utility::openPath( m_point,m_folderOpener,m_env,this,x,y ) ;
+	utility::openPath( m_point,m_folderOpener,this,x,y ) ;
 }
 
 void zuluMount::openMountPointPath( QString m )
@@ -1329,7 +1344,7 @@ void zuluMount::unmount( const QString& e )
 
 				DialogMsg m( this ) ;
 
-				QString z = r.output() ;
+				QString z = r.stdOut() ;
 
 				z.replace( tr( "ERROR: " ),"" ) ;
 				z.replace( "ERROR: ","" ) ;
@@ -1531,15 +1546,6 @@ void zuluMount::enableAll()
 	m_ui->pbmount->setEnabled( true ) ;
 	m_ui->tableWidget->setFocus() ;
 	m_ui->pbunlockcryptfs->setEnabled( true ) ;
-
-	if( utility::runningInMixedMode() ){
-
-		/*
-		 * FUSE related permission issues prevents us from currently supporting
-		 * unlocking encfs volumes when running in mixed mode.
-		 */
-		m_ui->pbunlockcryptfs->setEnabled( false ) ;
-	}
 }
 
 void zuluMount::enableAll_1()
@@ -1558,6 +1564,11 @@ bool zuluMount::autoMount()
 
 zuluMount::~zuluMount()
 {
+	if( !m_ui ){
+
+		return ;
+	}
+
 	QFile f( utility::homePath() + zuluMOUNT_AUTOPATH ) ;
 
 	if( m_autoMountAction ){
